/* (C) 2022 YiRing, Inc. */
package com.yiring.app.constant;

import com.yiring.app.exception.CodeException;
import com.yiring.common.core.I18n;
import io.swagger.annotations.ApiModel;
import org.jetbrains.annotations.PropertyKey;

/**
 * 业务状态码
 * eg: <code>throw Code.$100000.exception()</code>
 *
 * @author Jim
 * @version 0.1
 * 2022/3/25 9:23
 */

@SuppressWarnings({ "unused" })
@ApiModel("业务状态码")
public enum Code {
    SUCCESS(0, "Code.0"),

    // =============================================================

    // TODO: 扩展业务状态
    // eg:
    //     100001: 用户被禁止登录

    $100000(100000, "Code.100000"),
    $100001(100001, "Code.100001"),
    $100002(100002, "Code.100002")
    // =============================================================
    ;

    private final int value;
    private final String reasonPhrase;

    Code(int value, @PropertyKey(resourceBundle = I18n.RESOURCE_BUNDLE) String reasonPhrase) {
        this.value = value;
        this.reasonPhrase = reasonPhrase;
    }

    /**
     * Return the integer value of this status code.
     */
    public int value() {
        return this.value;
    }

    /**
     * Return the reason phrase of this status code.
     */
    public String reason() {
        return this.reasonPhrase;
    }

    /**
     * 业务代码异常
     */
    public CodeException exception() {
        return new CodeException(this);
    }

    /**
     * 暴露异常
     */
    public void expose() throws CodeException {
        throw exception();
    }
}
