/* (C) 2021 YiRing, Inc. */
package com.yiring.app.web.example;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.app.constant.Code;
import com.yiring.app.domain.user.UserExtension;
import com.yiring.app.domain.user.UserExtensionRepository;
import com.yiring.app.vo.user.UserExtensionVo;
import com.yiring.auth.annotation.AuthIgnore;
import com.yiring.auth.domain.user.User;
import com.yiring.auth.util.Auths;
import com.yiring.common.core.I18n;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.PageParam;
import com.yiring.common.util.Commons;
import com.yiring.common.util.FileUtils;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import javax.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.domain.Example;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 示例接口
 *
 * @author Jim
 */

@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@ApiSupport(order = 0)
@Api(tags = "示例", description = "Example")
@RequestMapping("/example/")
@RestController
@RequiredArgsConstructor
public class ExampleController {

    final I18n i18n;
    final Auths auths;
    final UserExtensionRepository userExtensionRepository;

    @GetMapping
    public Result<String> hello() {
        return Result.ok("example.hello");
    }

    /**
     * 测试失败自定义状态信息输出
     */
    @GetMapping("fail")
    public Result<String> fail() {
        throw Code.$100000.exception();
    }

    @SaCheckLogin
    @GetMapping("page")
    public Result<PageVo<String>> page(@Validated PageParam pageParam) {
        log.info("PageParam: {}", pageParam);

        String text = i18n.get("example.hello");
        List<String> data = Arrays.asList(text.split(" "));
        PageVo<String> vo = PageVo.build(data, data.size());
        return Result.ok(vo);
    }

    @AuthIgnore
    @SneakyThrows
    @ApiOperation(value = "download", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("download")
    public void download(HttpServletResponse response) {
        ClassPathResource resource = new ClassPathResource("static/cat.jpg");
        FileUtils.download(response, resource.getFile());
    }

    @SaCheckLogin
    @ApiOperation("查询用户属性")
    @GetMapping("findUserExtensionInfo")
    public Result<UserExtensionVo> findUserExtensionInfo() {
        User user = auths.getLoginUser();
        Optional<UserExtension> optional = userExtensionRepository.findOne(Example.of(new UserExtension(user)));
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        UserExtension ext = optional.get();
        UserExtensionVo vo = Commons.transform(ext, UserExtensionVo.class);
        return Result.ok(vo);
    }
}
